require "defines"
require "library/utilities"
---[[

baseName = "stack-assembler"

names = 
{
	stackerNumbers =
	{
		baseName .. "-1",
		baseName .. "-2",
		baseName .. "-3",
	},
	stacker = {}
}

for i=1, #names.stackerNumbers
do
	names.stacker[i] = 
	{
		n = names.stackerNumbers[i] .. "-north",
		s = names.stackerNumbers[i] .. "-south",
		e = names.stackerNumbers[i] .. "-east",
		w = names.stackerNumbers[i] .. "-west",	
	}
end

local nameList = {}
for i=1, #names.stacker
do
	table.insert(nameList, names.stacker[i].n)
	table.insert(nameList, names.stacker[i].s)
	table.insert(nameList, names.stacker[i].e)
	table.insert(nameList, names.stacker[i].w)
end

stackers = findEntitiesInGeneratedChunks(nameList)

function fill(stacker, targetArea)
	items = game.findentitiesfiltered
	{
		area = targetArea,
		name = "item-on-ground"
	}
	for i=1, #items
	do
		if stacker.caninsert{name = items[i].stack.name, count = items[i].stack.count}
		then
			stacker.insert{name = items[i].stack.name, count = items[i].stack.count}
			items[i].destroy()
		end
	end
end

function empty(stacker, target)
	target1 = moveTowards(target, right(stacker.direction), .25)
	target2 = moveTowards(target, left(stacker.direction), .25)
	inv = stacker.getoutputinventory()
	if inv.isempty() then return end
	item = itemName(inv.getcontents())
	if game.canplaceentity
	{
		name = "item-on-ground",
		position = target1,
		force = game.player.force,
		stack = {name = item, count = 1}
	}
	and #game.findentitiesfiltered{name = "item-on-ground", area = areaAround(target1, .1)} == 0
	then
		inv.remove{name = item, count = 1}
		game.createentity
		{
			name = "item-on-ground",
			position = target1,
			force = game.player.force,
			stack = {name = item, count = 1}
		}
	elseif game.canplaceentity
	{
		name = "item-on-ground",
		position = target2,
		force = game.player.force,
		stack = {name = item, count = 1}
	}
	and #game.findentitiesfiltered{name = "item-on-ground", area = areaAround(target2, .1)} == 0
	then
		inv.remove{name = item, count = 1}
		game.createentity
		{
			name = "item-on-ground",
			position = target2,
			force = game.player.force,
			stack = {name = item, count = 1}
		}
	end
end

function itemName(list)
	if list["iron-plate"] then key = "iron-plate"
	elseif list["copper-plate"] then key = "copper-plate"
	elseif list["steel-plate"] then key = "steel-plate"
	elseif list["coal"] then key = "coal"
	elseif list["plastic"] then key = "plastic"
	elseif list["stone-brick"] then key = "stone-brick"
	elseif list["stacked-iron-plates"] then key = "stacked-iron-plates"
	elseif list["stacked-copper-plates"] then key = "stacked-copper-plates"
	elseif list["stacked-steel-plates"] then key = "stacked-steel-plates"
	elseif list["stacked-iron-ore"] then key = "stacked-iron-ore"
	elseif list["stacked-copper-ore"] then key = "stacked-copper-ore"
	elseif list["stacked-coal"] then key = "stacked-coal"
	elseif list["stacked-plastic"] then key = "stacked-plastic"
	elseif list["stacked-stone-bricks"] then key = "stacked-stone-bricks"
	end
	return key
end

function stacking()
	for i = 1, #stackers
	do
		stacker = stackers[i]
		fill(stacker, areaAround(moveTowards(stacker.position, math.fmod(stacker.direction+4,8), 0.25), 0.25))
		empty(stacker, moveTowards(stacker.position, stacker.direction, .6))
	end
end

function whichStacker(entity)
	for i=1, #names.stackerNumbers
	do
		if string.find(entity.name, names.stackerNumbers[i], 0, true)
		then
			return i
		end
	end
end

function updateDirection(oldStacker)
	dir = oldStacker.direction
	if dir == defines.direction.north then newName = names.stacker[whichStacker(oldStacker)].n
	elseif dir == defines.direction.south then newName = names.stacker[whichStacker(oldStacker)].s
	elseif dir == defines.direction.east then newName = names.stacker[whichStacker(oldStacker)].e
	elseif dir == defines.direction.west then newName = names.stacker[whichStacker(oldStacker)].w end
	
	local newStacker =
	{
		name = newName,
		position = oldStacker.position,
		direction = dir,
		force = oldStacker.force,
	}
	place = getListPlace(oldStacker)
	local stacks = {}
	if oldStacker.recipe
	then
		rec = oldStacker.recipe
		newStacker.recipe = rec.name
		inv = {oldStacker.getinventory(2), oldStacker.getinventory(3), oldStacker.getinventory(4)}
		stacks[1] = 
		{
			name = rec.ingredients[1].name,
			count = inv[1].getitemcount(rec.ingredients[1].name)
		}
		stacks[2] =
		{
			name = rec.products[1].name,
			count = inv[2].getitemcount(rec.products[1].name)
		}
		if not inv[3].isempty()
		then
			stacks = mergeTables(stacks, getModules(inv[3]))
		end
	end
	oldStacker.destroy()
	stackers[place] = game.createentity(newStacker)
	for i=1, #stacks
	do
		if stacks[i].count > 0 then stackers[place].insert(stacks[i]) end
	end
	
end

function getModules(inv)
	cont = inv.getcontents()
	local stacks = {}
	mods = 0
	if cont["speed-module"]
	then
		mods = mods+1
		stacks[mods] = {name = "speed-module", count = cont["speed-module"]}
	end
	if cont["speed-module-2"]
	then
		mods = mods+1
		stacks[mods] = {name = "speed-module-2", count = cont["speed-module-2"]}
	end
	if cont["speed-module-3"]
	then
		mods = mods+1
		stacks[mods] = {name = "speed-module-3", count = cont["speed-module-3"]}
	end
	if cont["effectivity-module"]
	then
		mods = mods+1
		stacks[mods] = {name = "effectivity-module", count = cont["effectivity-module"]}
	end
	if cont["effectivity-module-2"]
	then
		mods = mods+1
		stacks[mods] = {name = "effectivity-module-2", count = cont["effectivity-module-2"]}
	end
	if cont["effectivity-module-3"]
	then
		mods = mods+1
		stacks[mods] = {name = "effectivity-module-3", count = cont["effectivity-module-3"]}
	end
	if cont["productivity-module"]
	then
		mods = mods+1
		stacks[mods] = {name = "productivity-module", count = cont["productivity-module"]}
	end
	if cont["productivity-module-2"]
	then
		mods = mods+1
		stacks[mods] = {name = "productivity-module-2", count = cont["productivity-module-2"]}
	end
	if cont["productivity-module-3"]
	then
		mods = mods+1
		stacks[mods] = {name = "productivity-module-3", count = cont["productivity-module-3"]}
	end
	return stacks
end

function getListPlace(entity)
	for i=1, #stackers
	do
		if stackers[i].equals(entity)
		then
			return i
		end
	end
end

function removeFromTable(entity)
	table.remove(stackers, getListPlace(entity))
end

function isStacker(entity)
	if string.find(entity.name, baseName, 0, true)
	then
		return true
	end
	return false
end

game.onevent(defines.events,
	function(event)
		if event.name == defines.events.ontick and getTime()>5
		then
			stepTimers()
			stacking()
		elseif event.name == defines.events.onbuiltentity and isStacker(event.createdentity)
		then
			local ent = event.createdentity
			table.insert(stackers, ent)
			if not ent.direction == defines.direction.north
			then
				ent.direction = defines.direction.north
			end
		elseif (event.name == defines.events.onentitydied or event.name == defines.events.onpreplayermineditem) and isStacker(event.entity)
		then
			removeFromTable(event.entity)
		elseif event.name == defines.events.onplayerrotatedentity and isStacker(event.entity)
		then
			updateDirection(event.entity)
		end
	end
)
 --]]